package java.time.format;

import java.text.ParseException;
import java.text.FieldPosition;
import java.util.HashMap;
import java.time.temporal.IsoFields;
import java.time.temporal.ChronoField;
import java.text.Format;
import java.text.ParsePosition;
import java.io.IOException;
import java.time.DateTimeException;
import java.time.temporal.TemporalAccessor;
import java.util.Collections;
import java.util.HashSet;
import java.util.Arrays;
import java.time.chrono.IsoChronology;
import java.util.Objects;
import java.time.Period;
import java.time.temporal.TemporalQuery;
import java.time.ZoneId;
import java.time.chrono.Chronology;
import java.time.temporal.TemporalField;
import java.util.Set;
import java.util.Locale;

public final class DateTimeFormatter
{
    private final DateTimeFormatterBuilder.CompositePrinterParser printerParser;
    private final Locale locale;
    private final DecimalStyle decimalStyle;
    private final ResolverStyle resolverStyle;
    private final Set<TemporalField> resolverFields;
    private final Chronology chrono;
    private final ZoneId zone;
    public static final DateTimeFormatter ISO_LOCAL_DATE;
    public static final DateTimeFormatter ISO_OFFSET_DATE;
    public static final DateTimeFormatter ISO_DATE;
    public static final DateTimeFormatter ISO_LOCAL_TIME;
    public static final DateTimeFormatter ISO_OFFSET_TIME;
    public static final DateTimeFormatter ISO_TIME;
    public static final DateTimeFormatter ISO_LOCAL_DATE_TIME;
    public static final DateTimeFormatter ISO_OFFSET_DATE_TIME;
    public static final DateTimeFormatter ISO_ZONED_DATE_TIME;
    public static final DateTimeFormatter ISO_DATE_TIME;
    public static final DateTimeFormatter ISO_ORDINAL_DATE;
    public static final DateTimeFormatter ISO_WEEK_DATE;
    public static final DateTimeFormatter ISO_INSTANT;
    public static final DateTimeFormatter BASIC_ISO_DATE;
    public static final DateTimeFormatter RFC_1123_DATE_TIME;
    private static final TemporalQuery<Period> PARSED_EXCESS_DAYS;
    private static final TemporalQuery<Boolean> PARSED_LEAP_SECOND;
    
    public static DateTimeFormatter ofPattern(String s) {
        return new DateTimeFormatterBuilder().appendPattern(s).toFormatter();
    }
    
    public static DateTimeFormatter ofPattern(String s, Locale locale) {
        return new DateTimeFormatterBuilder().appendPattern(s).toFormatter(locale);
    }
    
    public static DateTimeFormatter ofLocalizedDate(FormatStyle obj) {
        Objects.requireNonNull(obj, "dateStyle");
        return new DateTimeFormatterBuilder().appendLocalized(obj, null).toFormatter(ResolverStyle.SMART, IsoChronology.INSTANCE);
    }
    
    public static DateTimeFormatter ofLocalizedTime(FormatStyle obj) {
        Objects.requireNonNull(obj, "timeStyle");
        return new DateTimeFormatterBuilder().appendLocalized(null, obj).toFormatter(ResolverStyle.SMART, IsoChronology.INSTANCE);
    }
    
    public static DateTimeFormatter ofLocalizedDateTime(FormatStyle obj) {
        Objects.requireNonNull(obj, "dateTimeStyle");
        return new DateTimeFormatterBuilder().appendLocalized(obj, obj).toFormatter(ResolverStyle.SMART, IsoChronology.INSTANCE);
    }
    
    public static DateTimeFormatter ofLocalizedDateTime(FormatStyle obj, FormatStyle obj2) {
        Objects.requireNonNull(obj, "dateStyle");
        Objects.requireNonNull(obj2, "timeStyle");
        return new DateTimeFormatterBuilder().appendLocalized(obj, obj2).toFormatter(ResolverStyle.SMART, IsoChronology.INSTANCE);
    }
    
    public static final TemporalQuery<Period> parsedExcessDays() {
        return DateTimeFormatter.PARSED_EXCESS_DAYS;
    }
    
    public static final TemporalQuery<Boolean> parsedLeapSecond() {
        return DateTimeFormatter.PARSED_LEAP_SECOND;
    }
    
    DateTimeFormatter(DateTimeFormatterBuilder.CompositePrinterParser obj, Locale obj2, DecimalStyle obj3, ResolverStyle obj4, Set<TemporalField> resolverFields, Chronology chrono, ZoneId zone) {
        this.printerParser = (DateTimeFormatterBuilder.CompositePrinterParser)Objects.requireNonNull(obj, "printerParser");
        this.resolverFields = resolverFields;
        this.locale = (Locale)Objects.requireNonNull(obj2, "locale");
        this.decimalStyle = (DecimalStyle)Objects.requireNonNull(obj3, "decimalStyle");
        this.resolverStyle = (ResolverStyle)Objects.requireNonNull(obj4, "resolverStyle");
        this.chrono = chrono;
        this.zone = zone;
    }
    
    public Locale getLocale() {
        return this.locale;
    }
    
    public DateTimeFormatter withLocale(Locale obj) {
        if (this.locale.equals(obj)) {
            return this;
        }
        return new DateTimeFormatter(this.printerParser, obj, this.decimalStyle, this.resolverStyle, this.resolverFields, this.chrono, this.zone);
    }
    
    public DecimalStyle getDecimalStyle() {
        return this.decimalStyle;
    }
    
    public DateTimeFormatter withDecimalStyle(DecimalStyle decimalStyle) {
        if (this.decimalStyle.equals(decimalStyle)) {
            return this;
        }
        return new DateTimeFormatter(this.printerParser, this.locale, decimalStyle, this.resolverStyle, this.resolverFields, this.chrono, this.zone);
    }
    
    public Chronology getChronology() {
        return this.chrono;
    }
    
    public DateTimeFormatter withChronology(Chronology b) {
        if (Objects.equals(this.chrono, b)) {
            return this;
        }
        return new DateTimeFormatter(this.printerParser, this.locale, this.decimalStyle, this.resolverStyle, this.resolverFields, b, this.zone);
    }
    
    public ZoneId getZone() {
        return this.zone;
    }
    
    public DateTimeFormatter withZone(ZoneId b) {
        if (Objects.equals(this.zone, b)) {
            return this;
        }
        return new DateTimeFormatter(this.printerParser, this.locale, this.decimalStyle, this.resolverStyle, this.resolverFields, this.chrono, b);
    }
    
    public ResolverStyle getResolverStyle() {
        return this.resolverStyle;
    }
    
    public DateTimeFormatter withResolverStyle(ResolverStyle resolverStyle) {
        Objects.requireNonNull(resolverStyle, "resolverStyle");
        if (Objects.equals(this.resolverStyle, resolverStyle)) {
            return this;
        }
        return new DateTimeFormatter(this.printerParser, this.locale, this.decimalStyle, resolverStyle, this.resolverFields, this.chrono, this.zone);
    }
    
    public Set<TemporalField> getResolverFields() {
        return this.resolverFields;
    }
    
    public DateTimeFormatter withResolverFields(TemporalField... a) {
        Set<Object> unmodifiableSet = null;
        if (a != null) {
            unmodifiableSet = Collections.unmodifiableSet(new HashSet(Arrays.asList(a)));
        }
        if (Objects.equals(this.resolverFields, unmodifiableSet)) {
            return this;
        }
        return new DateTimeFormatter(this.printerParser, this.locale, this.decimalStyle, this.resolverStyle, unmodifiableSet, this.chrono, this.zone);
    }
    
    public DateTimeFormatter withResolverFields(Set<TemporalField> unmodifiableSet) {
        if (Objects.equals(this.resolverFields, unmodifiableSet)) {
            return this;
        }
        if (unmodifiableSet != null) {
            unmodifiableSet = Collections.unmodifiableSet(new HashSet(unmodifiableSet));
        }
        return new DateTimeFormatter(this.printerParser, this.locale, this.decimalStyle, this.resolverStyle, unmodifiableSet, this.chrono, this.zone);
    }
    
    public String format(TemporalAccessor temporalAccessor) {
        StringBuilder sb = new StringBuilder(32);
        this.formatTo(temporalAccessor, sb);
        return sb.toString();
    }
    
    public void formatTo(TemporalAccessor obj, Appendable obj2) {
        Objects.requireNonNull(obj, "temporal");
        Objects.requireNonNull(obj2, "appendable");
        try {
            DateTimePrintContext dateTimePrintContext = new DateTimePrintContext(obj, this);
            if ((obj2 instanceof StringBuilder)) {
                this.printerParser.format(dateTimePrintContext, (StringBuilder)obj2);
            }
            else {
                StringBuilder sb = new StringBuilder(32);
                this.printerParser.format(dateTimePrintContext, sb);
                obj2.append(sb);
            }
        }
        catch (IOException cause) {
            throw new DateTimeException(cause.getMessage(), cause);
        }
    }
    
    public TemporalAccessor parse(CharSequence obj) {
        Objects.requireNonNull(obj, "text");
        try {
            return this.parseResolved0(obj, null);
        }
        catch (DateTimeParseException ex) {
            throw ex;
        }
        catch (RuntimeException ex2) {
            throw this.createError(obj, ex2);
        }
    }
    
    public TemporalAccessor parse(CharSequence obj, ParsePosition obj2) {
        Objects.requireNonNull(obj, "text");
        Objects.requireNonNull(obj2, "position");
        try {
            return this.parseResolved0(obj, obj2);
        }
        catch (DateTimeParseException | IndexOutOfBoundsException ex) {
            throw ex;
        }
        catch (RuntimeException ex2) {
            throw this.createError(obj, ex2);
        }
    }
    
    public <T> T parse(CharSequence obj, TemporalQuery<T> temporalQuery) {
        Objects.requireNonNull(obj, "text");
        Objects.requireNonNull(temporalQuery, "query");
        try {
            return this.parseResolved0(obj, null).query(temporalQuery);
        }
        catch (DateTimeParseException ex) {
            throw ex;
        }
        catch (RuntimeException ex2) {
            throw this.createError(obj, ex2);
        }
    }
    
    public TemporalAccessor parseBest(CharSequence obj, TemporalQuery<?>... obj2) {
        Objects.requireNonNull(obj, "text");
        Objects.requireNonNull(obj2, "queries");
        if (obj2.length < 2) {
            throw new IllegalArgumentException("At least two queries must be specified");
        }
        try {
            TemporalAccessor resolved0 = this.parseResolved0(obj, null);
            int length = obj2.length;
            int i = 0;
            while (i < length) {
                TemporalQuery<?> query = obj2[i];
                try {
                    return (TemporalAccessor)resolved0.query(query);
                }
                catch (RuntimeException ex) {
                    ++i;
                }
            }
            throw new DateTimeException("Unable to convert parsed text using any of the specified queries");
        }
        catch (DateTimeParseException ex2) {
            throw ex2;
        }
        catch (RuntimeException ex3) {
            throw this.createError(obj, ex3);
        }
    }
    
    private DateTimeParseException createError(CharSequence charSequence, RuntimeException ex) {
        String str;
        if (charSequence.length() > 64) {
            str = charSequence.subSequence(0, 64).toString() + "...";
        }
        else {
            str = charSequence.toString();
        }
        return new DateTimeParseException("Text '" + str + "' could not be parsed: " + ex.getMessage(), charSequence, 0, ex);
    }
    
    private TemporalAccessor parseResolved0(CharSequence charSequence, ParsePosition parsePosition) {
        ParsePosition parsePosition2 = (parsePosition != null) ? parsePosition : new ParsePosition(0);
        DateTimeParseContext unresolved0 = this.parseUnresolved0(charSequence, parsePosition2);
        if (unresolved0 == null || parsePosition2.getErrorIndex() >= 0 || (parsePosition == null && parsePosition2.getIndex() < charSequence.length())) {
            String s;
            if (charSequence.length() > 64) {
                s = charSequence.subSequence(0, 64).toString() + "...";
            }
            else {
                s = charSequence.toString();
            }
            if (parsePosition2.getErrorIndex() >= 0) {
                throw new DateTimeParseException("Text '" + s + "' could not be parsed at index " + parsePosition2.getErrorIndex(), charSequence, parsePosition2.getErrorIndex());
            }
            throw new DateTimeParseException("Text '" + s + "' could not be parsed, unparsed text found at index " + parsePosition2.getIndex(), charSequence, parsePosition2.getIndex());
        }
        else {
            return unresolved0.toResolved(this.resolverStyle, this.resolverFields);
        }
    }
    
    public TemporalAccessor parseUnresolved(CharSequence charSequence, ParsePosition parsePosition) {
        DateTimeParseContext unresolved0 = this.parseUnresolved0(charSequence, parsePosition);
        if (unresolved0 == null) {
            return null;
        }
        return unresolved0.toUnresolved();
    }
    
    private DateTimeParseContext parseUnresolved0(CharSequence obj, ParsePosition obj2) {
        Objects.requireNonNull(obj, "text");
        Objects.requireNonNull(obj2, "position");
        DateTimeParseContext dateTimeParseContext = new DateTimeParseContext(this);
        int parse = this.printerParser.parse(dateTimeParseContext, obj, obj2.getIndex());
        if (parse < 0) {
            obj2.setErrorIndex(~parse);
            return null;
        }
        obj2.setIndex(parse);
        return dateTimeParseContext;
    }
    
    DateTimeFormatterBuilder.CompositePrinterParser toPrinterParser(boolean b) {
        return this.printerParser.withOptional(b);
    }
    
    public Format toFormat() {
        return new DateTimeFormatter.ClassicFormat(this, null);
    }
    
    public Format toFormat(TemporalQuery<?> obj) {
        Objects.requireNonNull(obj, "parseQuery");
        return new DateTimeFormatter.ClassicFormat(this, obj);
    }
    
    @Override
    public String toString() {
        String string = this.printerParser.toString();
        return string.startsWith("[") ? string : string.substring(1, string.length() - 1);
    }
    
    static TemporalAccessor access$000(DateTimeFormatter dateTimeFormatter, CharSequence charSequence, ParsePosition parsePosition) {
        return dateTimeFormatter.parseResolved0(charSequence, parsePosition);
    }
    
    static DateTimeParseContext access$100(DateTimeFormatter dateTimeFormatter, CharSequence charSequence, ParsePosition parsePosition) {
        return dateTimeFormatter.parseUnresolved0(charSequence, parsePosition);
    }
    
    static {
        ISO_LOCAL_DATE = new DateTimeFormatterBuilder().appendValue(ChronoField.YEAR, 4, 10, SignStyle.EXCEEDS_PAD).appendLiteral('-').appendValue(ChronoField.MONTH_OF_YEAR, 2).appendLiteral('-').appendValue(ChronoField.DAY_OF_MONTH, 2).toFormatter(ResolverStyle.STRICT, IsoChronology.INSTANCE);
        ISO_OFFSET_DATE = new DateTimeFormatterBuilder().parseCaseInsensitive().append(DateTimeFormatter.ISO_LOCAL_DATE).appendOffsetId().toFormatter(ResolverStyle.STRICT, IsoChronology.INSTANCE);
        ISO_DATE = new DateTimeFormatterBuilder().parseCaseInsensitive().append(DateTimeFormatter.ISO_LOCAL_DATE).optionalStart().appendOffsetId().toFormatter(ResolverStyle.STRICT, IsoChronology.INSTANCE);
        ISO_LOCAL_TIME = new DateTimeFormatterBuilder().appendValue(ChronoField.HOUR_OF_DAY, 2).appendLiteral(':').appendValue(ChronoField.MINUTE_OF_HOUR, 2).optionalStart().appendLiteral(':').appendValue(ChronoField.SECOND_OF_MINUTE, 2).optionalStart().appendFraction(ChronoField.NANO_OF_SECOND, 0, 9, true).toFormatter(ResolverStyle.STRICT, null);
        ISO_OFFSET_TIME = new DateTimeFormatterBuilder().parseCaseInsensitive().append(DateTimeFormatter.ISO_LOCAL_TIME).appendOffsetId().toFormatter(ResolverStyle.STRICT, null);
        ISO_TIME = new DateTimeFormatterBuilder().parseCaseInsensitive().append(DateTimeFormatter.ISO_LOCAL_TIME).optionalStart().appendOffsetId().toFormatter(ResolverStyle.STRICT, null);
        ISO_LOCAL_DATE_TIME = new DateTimeFormatterBuilder().parseCaseInsensitive().append(DateTimeFormatter.ISO_LOCAL_DATE).appendLiteral('T').append(DateTimeFormatter.ISO_LOCAL_TIME).toFormatter(ResolverStyle.STRICT, IsoChronology.INSTANCE);
        ISO_OFFSET_DATE_TIME = new DateTimeFormatterBuilder().parseCaseInsensitive().append(DateTimeFormatter.ISO_LOCAL_DATE_TIME).appendOffsetId().toFormatter(ResolverStyle.STRICT, IsoChronology.INSTANCE);
        ISO_ZONED_DATE_TIME = new DateTimeFormatterBuilder().append(DateTimeFormatter.ISO_OFFSET_DATE_TIME).optionalStart().appendLiteral('[').parseCaseSensitive().appendZoneRegionId().appendLiteral(']').toFormatter(ResolverStyle.STRICT, IsoChronology.INSTANCE);
        ISO_DATE_TIME = new DateTimeFormatterBuilder().append(DateTimeFormatter.ISO_LOCAL_DATE_TIME).optionalStart().appendOffsetId().optionalStart().appendLiteral('[').parseCaseSensitive().appendZoneRegionId().appendLiteral(']').toFormatter(ResolverStyle.STRICT, IsoChronology.INSTANCE);
        ISO_ORDINAL_DATE = new DateTimeFormatterBuilder().parseCaseInsensitive().appendValue(ChronoField.YEAR, 4, 10, SignStyle.EXCEEDS_PAD).appendLiteral('-').appendValue(ChronoField.DAY_OF_YEAR, 3).optionalStart().appendOffsetId().toFormatter(ResolverStyle.STRICT, IsoChronology.INSTANCE);
        ISO_WEEK_DATE = new DateTimeFormatterBuilder().parseCaseInsensitive().appendValue(IsoFields.WEEK_BASED_YEAR, 4, 10, SignStyle.EXCEEDS_PAD).appendLiteral("-W").appendValue(IsoFields.WEEK_OF_WEEK_BASED_YEAR, 2).appendLiteral('-').appendValue(ChronoField.DAY_OF_WEEK, 1).optionalStart().appendOffsetId().toFormatter(ResolverStyle.STRICT, IsoChronology.INSTANCE);
        ISO_INSTANT = new DateTimeFormatterBuilder().parseCaseInsensitive().appendInstant().toFormatter(ResolverStyle.STRICT, null);
        BASIC_ISO_DATE = new DateTimeFormatterBuilder().parseCaseInsensitive().appendValue(ChronoField.YEAR, 4).appendValue(ChronoField.MONTH_OF_YEAR, 2).appendValue(ChronoField.DAY_OF_MONTH, 2).optionalStart().appendOffset("+HHMMss", "Z").toFormatter(ResolverStyle.STRICT, IsoChronology.INSTANCE);
        HashMap<Long, String> hashMap = new HashMap();
        hashMap.put(Long.valueOf(1L), "Mon");
        hashMap.put(Long.valueOf(2L), "Tue");
        hashMap.put(Long.valueOf(3L), "Wed");
        hashMap.put(Long.valueOf(4L), "Thu");
        hashMap.put(Long.valueOf(5L), "Fri");
        hashMap.put(Long.valueOf(6L), "Sat");
        hashMap.put(Long.valueOf(7L), "Sun");
        HashMap<Long, String> hashMap2 = new HashMap();
        hashMap2.put(Long.valueOf(1L), "Jan");
        hashMap2.put(Long.valueOf(2L), "Feb");
        hashMap2.put(Long.valueOf(3L), "Mar");
        hashMap2.put(Long.valueOf(4L), "Apr");
        hashMap2.put(Long.valueOf(5L), "May");
        hashMap2.put(Long.valueOf(6L), "Jun");
        hashMap2.put(Long.valueOf(7L), "Jul");
        hashMap2.put(Long.valueOf(8L), "Aug");
        hashMap2.put(Long.valueOf(9L), "Sep");
        hashMap2.put(Long.valueOf(10L), "Oct");
        hashMap2.put(Long.valueOf(11L), "Nov");
        hashMap2.put(Long.valueOf(12L), "Dec");
        RFC_1123_DATE_TIME = new DateTimeFormatterBuilder().parseCaseInsensitive().parseLenient().optionalStart().appendText(ChronoField.DAY_OF_WEEK, hashMap).appendLiteral(", ").optionalEnd().appendValue(ChronoField.DAY_OF_MONTH, 1, 2, SignStyle.NOT_NEGATIVE).appendLiteral(' ').appendText(ChronoField.MONTH_OF_YEAR, hashMap2).appendLiteral(' ').appendValue(ChronoField.YEAR, 4).appendLiteral(' ').appendValue(ChronoField.HOUR_OF_DAY, 2).appendLiteral(':').appendValue(ChronoField.MINUTE_OF_HOUR, 2).optionalStart().appendLiteral(':').appendValue(ChronoField.SECOND_OF_MINUTE, 2).optionalEnd().appendLiteral(' ').appendOffset("+HHMM", "GMT").toFormatter(ResolverStyle.SMART, IsoChronology.INSTANCE);
        PARSED_EXCESS_DAYS = (temporalAccessor/* java.time.temporal.TemporalAccessor, */ -> {
            if ((temporalAccessor instanceof Parsed)) {
                return ((Parsed)temporalAccessor).excessDays;
            }
            return Period.ZERO;
        });
        PARSED_LEAP_SECOND = (temporalAccessor3/* java.time.temporal.TemporalAccessor, */ -> {
            if ((temporalAccessor3 instanceof Parsed)) {
                return Boolean.valueOf(((Parsed)temporalAccessor3).leapSecond);
            }
            return Boolean.FALSE;
        });
    }
    
    static class ClassicFormat extends Format
    {
        private final DateTimeFormatter formatter;
        private final TemporalQuery<?> parseType;
        
        public ClassicFormat(DateTimeFormatter formatter, TemporalQuery<?> parseType) {
            this.formatter = formatter;
            this.parseType = parseType;
        }
        
        @Override
        public StringBuffer format(Object obj, StringBuffer obj2, FieldPosition obj3) {
            Objects.requireNonNull(obj, "obj");
            Objects.requireNonNull(obj2, "toAppendTo");
            Objects.requireNonNull(obj3, "pos");
            if (!(obj instanceof TemporalAccessor)) {
                throw new IllegalArgumentException("Format target must implement TemporalAccessor");
            }
            obj3.setBeginIndex(0);
            obj3.setEndIndex(0);
            try {
                this.formatter.formatTo((TemporalAccessor)obj, obj2);
            }
            catch (RuntimeException cause) {
                throw new IllegalArgumentException(cause.getMessage(), cause);
            }
            return obj2;
        }
        
        @Override
        public Object parseObject(String obj) throws ParseException {
            Objects.requireNonNull(obj, "text");
            try {
                if (this.parseType == null) {
                    return DateTimeFormatter.access$000(this.formatter, obj, null);
                }
                return this.formatter.parse(obj, this.parseType);
            }
            catch (DateTimeParseException ex) {
                throw new ParseException(ex.getMessage(), ex.getErrorIndex());
            }
            catch (RuntimeException cause) {
                throw (ParseException)new ParseException(cause.getMessage(), 0).initCause(cause);
            }
        }
        
        @Override
        public Object parseObject(String obj, ParsePosition parsePosition) {
            Objects.requireNonNull(obj, "text");
            DateTimeParseContext access$100;
            try {
                access$100 = DateTimeFormatter.access$100(this.formatter, obj, parsePosition);
            }
            catch (IndexOutOfBoundsException ex) {
                if (parsePosition.getErrorIndex() < 0) {
                    parsePosition.setErrorIndex(0);
                }
                return null;
            }
            if (access$100 == null) {
                if (parsePosition.getErrorIndex() < 0) {
                    parsePosition.setErrorIndex(0);
                }
                return null;
            }
            else {
                try {
                    TemporalAccessor resolved = access$100.toResolved(this.formatter.resolverStyle, this.formatter.resolverFields);
                    if (this.parseType == null) {
                        return resolved;
                    }
                    return resolved.query(this.parseType);
                }
                catch (RuntimeException ex2) {
                    parsePosition.setErrorIndex(0);
                    return null;
                }
            }
        }
    }
}
